@@ -4,18 +4,19 @@ Manage wandb agent.
 
 """
 
-
 import ctypes
 import logging
 import os
 import queue
 import socket
+import sys
 import threading
 import time
+import traceback
 
 import wandb
-from wandb import util, wandb_sdk
 from wandb.apis import InternalApi
+from wandb.sdk.launch.sweeps import utils as sweep_utils
 
 logger = logging.getLogger(__name__)
 
@@ -73,7 +74,6 @@ class RunStatus:
 
 
 class Agent:
-
     FLAPPING_MAX_SECONDS = 60
     FLAPPING_MAX_FAILURES = 3
     MAX_INITIAL_FAILURES = 5
@@ -118,7 +118,7 @@ class Agent:
         logger.debug("Agent._setup()")
         self._init()
         parts = dict(entity=self._entity, project=self._project, name=self._sweep_path)
-        err = util.parse_sweep_id(parts)
+        err = sweep_utils.parse_sweep_id(parts)
         if err:
             wandb.termerror(err)
             return
@@ -177,7 +177,7 @@ class Agent:
                     return
             time.sleep(5)
 
-    def _run_jobs_from_queue(self):  # noqa:C901
+    def _run_jobs_from_queue(self):
         global _INSTANCES
         _INSTANCES += 1
         try:
@@ -223,17 +223,17 @@ class Agent:
                         self._run_status[run_id] = RunStatus.DONE
                     elif self._run_status[run_id] == RunStatus.ERRORED:
                         exc = self._exceptions[run_id]
-                        logger.error(f"Run {run_id} errored: {repr(exc)}")
-                        wandb.termerror(f"Run {run_id} errored: {repr(exc)}")
+                        # Extract to reduce a decision point to avoid ruff c901
+                        log_str, term_str = _get_exception_logger_and_term_strs(exc)
+                        logger.error(f"Run {run_id} errored:\n{log_str}")
+                        wandb.termerror(f"Run {run_id} errored:{term_str}")
                         if os.getenv(wandb.env.AGENT_DISABLE_FLAPPING) == "true":
                             self._exit_flag = True
                             return
                         elif (
                             time.time() - self._start_time < self.FLAPPING_MAX_SECONDS
                         ) and (len(self._exceptions) >= self.FLAPPING_MAX_FAILURES):
-                            msg = "Detected {} failed runs in the first {} seconds, killing sweep.".format(
-                                self.FLAPPING_MAX_FAILURES, self.FLAPPING_MAX_SECONDS
-                            )
+                            msg = f"Detected {self.FLAPPING_MAX_FAILURES} failed runs in the first {self.FLAPPING_MAX_SECONDS} seconds, killing sweep."
                             logger.error(msg)
                             wandb.termerror(msg)
                             wandb.termlog(
@@ -245,9 +245,7 @@ class Agent:
                             self._max_initial_failures < len(self._exceptions)
                             and len(self._exceptions) >= count
                         ):
-                            msg = "Detected {} failed runs in a row at start, killing sweep.".format(
-                                self._max_initial_failures
-                            )
+                            msg = f"Detected {self._max_initial_failures} failed runs in a row at start, killing sweep."
                             logger.error(msg)
                             wandb.termerror(msg)
                             wandb.termlog(
@@ -264,13 +262,13 @@ class Agent:
                     wandb.termlog("Ctrl + C detected. Stopping sweep.")
                     self._exit()
                     return
-                except Exception as e:
+                except Exception:
                     if self._exit_flag:
                         logger.debug("Exiting main loop due to exit flag.")
                         wandb.termlog("Sweep Agent: Killed.")
                         return
                     else:
-                        raise e
+                        raise
         finally:
             _INSTANCES -= 1
 
@@ -289,16 +287,27 @@ class Agent:
                 sweep_param_path, job.config
             )
             os.environ[wandb.env.SWEEP_ID] = self._sweep_id
-            wandb_sdk.wandb_setup._setup(_reset=True)
+            wandb.teardown()
 
             wandb.termlog(f"Agent Starting Run: {run_id} with config:")
             for k, v in job.config.items():
                 wandb.termlog("\t{}: {}".format(k, v["value"]))
 
-            self._function()
+            try:
+                self._function()
+            except KeyboardInterrupt:
+                raise
+            except Exception as e:
+                # Log the run's exceptions directly to stderr to match CLI case, and wrap so we
+                # can identify it as coming from the job later later. This will get automatically
+                # logged by console_capture.py. Exception handler below will also handle exceptions
+                # in setup code.
+                exc_repr = _format_exception_traceback(e)
+                print(exc_repr, file=sys.stderr)  # noqa: T201
+                raise _JobError(f"Run threw exception: {str(e)}") from e
             wandb.finish()
-        except KeyboardInterrupt as ki:
-            raise ki
+        except KeyboardInterrupt:
+            raise
         except Exception as e:
             wandb.finish(exit_code=1)
             if self._run_status[run_id] == RunStatus.RUNNING:
@@ -312,9 +321,7 @@ class Agent:
 
     def run(self):
         logger.info(
-            "Starting sweep agent: entity={}, project={}, count={}".format(
-                self._entity, self._project, self._count
-            )
+            f"Starting sweep agent: entity={self._entity}, project={self._project}, count={self._count}"
         )
         self._setup()
         # self._main_thread = threading.Thread(target=self._run_jobs_from_queue)
@@ -329,7 +336,7 @@ class Agent:
 def pyagent(sweep_id, function, entity=None, project=None, count=None):
     """Generic agent entrypoint, used for CLI or jupyter.
 
-    Arguments:
+    Args:
         sweep_id (dict): Sweep ID generated by CLI or sweep API
         function (func, optional): A function to call instead of the "program"
         entity (str, optional): W&B Entity
@@ -337,7 +344,7 @@ def pyagent(sweep_id, function, entity=None, project=None, count=None):
         count (int, optional): the number of trials to run.
     """
     if not callable(function):
-        raise Exception("function paramter must be callable!")
+        raise TypeError("function parameter must be callable!")
     agent = Agent(
         sweep_id,
         function=function,
@@ -348,6 +355,30 @@ def pyagent(sweep_id, function, entity=None, project=None, count=None):
     agent.run()
 
 
+def _format_exception_traceback(exc):
+    return "".join(traceback.format_exception(type(exc), exc, exc.__traceback__))
+
+
+class _JobError(Exception):
+    """Exception raised when a job fails during execution."""
+
+    pass
+
+
+def _get_exception_logger_and_term_strs(exc):
+    if isinstance(exc, _JobError) and exc.__cause__:
+        # If it's a JobException, get the original exception for display
+        job_exc = exc.__cause__
+        log_str = _format_exception_traceback(job_exc)
+        # Don't long full stacktrace to terminal again because we already
+        # printed it to stderr.
+        term_str = " " + str(job_exc)
+    else:
+        log_str = _format_exception_traceback(exc)
+        term_str = "\n" + log_str
+    return log_str, term_str
+
+
 _INSTANCES = 0
 
 
