@@ -1,13 +1,11 @@
 import json
 import logging
 import os
+import shutil
 import subprocess
 from typing import Any, Dict, List, Optional, Tuple, Union
 
-import requests
-from dockerpycreds.utils import find_executable  # type: ignore
-
-from wandb.docker import auth, www_authenticate
+from wandb.docker import names
 from wandb.errors import Error
 
 
@@ -46,7 +44,6 @@ class DockerError(Error):
 entrypoint = os.path.join(
     os.path.dirname(os.path.abspath(__file__)), "wandb-entrypoint.sh"
 )
-auth_config = auth.load_config()
 log = logging.getLogger(__name__)
 
 
@@ -62,7 +59,7 @@ def shell(cmd: List[str]) -> Optional[str]:
             .strip()
         )
     except subprocess.CalledProcessError as e:
-        print(e)
+        print(e)  # noqa: T201
         return None
 
 
@@ -74,7 +71,7 @@ def is_buildx_installed() -> bool:
     global _buildx_installed
     if _buildx_installed is not None:
         return _buildx_installed  # type: ignore
-    if not find_executable("docker"):
+    if not shutil.which("docker"):
         _buildx_installed = False
     else:
         help_output = shell(["buildx", "--help"])
@@ -82,8 +79,31 @@ def is_buildx_installed() -> bool:
     return _buildx_installed
 
 
-def build(tags: List[str], file: str, context_path: str) -> str:
-    command = ["buildx", "build"] if is_buildx_installed() else ["build"]
+def is_docker_installed() -> bool:
+    """Return `True` if docker is installed and working, else `False`."""
+    try:
+        # Run the docker --version command
+        result = subprocess.run(
+            ["docker", "--version"],
+            capture_output=True,
+        )
+        if result.returncode == 0:
+            return True
+        else:
+            return False
+    except FileNotFoundError:
+        # If docker command is not found
+        return False
+
+
+def build(
+    tags: List[str], file: str, context_path: str, platform: Optional[str] = None
+) -> str:
+    use_buildx = is_buildx_installed()
+    command = ["buildx", "build"] if use_buildx else ["build"]
+    command += ["--load"] if should_add_load_argument(platform) and use_buildx else []
+    if platform:
+        command += ["--platform", platform]
     build_tags = []
     for tag in tags:
         build_tags += ["-t", tag]
@@ -94,6 +114,14 @@ def build(tags: List[str], file: str, context_path: str) -> str:
     return stdout
 
 
+def should_add_load_argument(platform: Optional[str]) -> bool:
+    # the load option does not work when multiple platforms are specified:
+    # https://github.com/docker/buildx/issues/59
+    if platform is None or (platform and "," not in platform):
+        return True
+    return False
+
+
 def run_command_live_output(args: List[Any]) -> str:
     with subprocess.Popen(
         args,
@@ -109,12 +137,12 @@ def run_command_live_output(args: List[Any]) -> str:
                 break
             index = chunk.find(b"\r")
             if index != -1:
-                print(chunk.decode(), end="")
+                print(chunk.decode(), end="")  # noqa: T201
             else:
                 stdout += chunk.decode()
-                print(chunk.decode(), end="\r")
+                print(chunk.decode(), end="\r")  # noqa: T201
 
-        print(stdout)
+        print(stdout)  # noqa: T201
 
     return_code = process.wait()
     if return_code != 0:
@@ -172,7 +200,7 @@ def default_image(gpu: bool = False) -> str:
     tag = "all"
     if not gpu:
         tag += "-cpu"
-    return "wandb/deepo:%s" % tag
+    return f"wandb/deepo:{tag}"
 
 
 def parse_repository_tag(repo_name: str) -> Tuple[str, Optional[str]]:
@@ -187,90 +215,39 @@ def parse_repository_tag(repo_name: str) -> Tuple[str, Optional[str]]:
 
 def parse(image_name: str) -> Tuple[str, str, str]:
     repository, tag = parse_repository_tag(image_name)
-    registry, repo_name = auth.resolve_repository_name(repository)
+    registry, repo_name = names.resolve_repository_name(repository)
     if registry == "docker.io":
         registry = "index.docker.io"
     return registry, repo_name, (tag or "latest")
 
 
-def auth_token(registry: str, repo: str) -> Dict[str, str]:
-    """Make a request to the root of a v2 docker registry to get the auth url.
-
-    Always returns a dictionary, if there's no token key we couldn't authenticate
-    """
-    # TODO: Cache tokens?
-    auth_info = auth_config.resolve_authconfig(registry)
-    if auth_info:
-        normalized = {k.lower(): v for k, v in auth_info.items()}
-        normalized_auth_info = (
-            normalized.get("username"),
-            normalized.get("password"),
-        )
-    else:
-        normalized_auth_info = None
-    response = requests.get(f"https://{registry}/v2/", timeout=3)
-    if response.headers.get("www-authenticate"):
-        try:
-            info: Dict = www_authenticate.parse(response.headers["www-authenticate"])
-        except ValueError:
-            info = {}
-    else:
-        log.error(
-            "Received {} when attempting to authenticate with {}".format(
-                response, registry
-            )
-        )
-        info = {}
-    if info.get("bearer"):
-        res = requests.get(
-            info["bearer"]["realm"]
-            + "?service={}&scope=repository:{}:pull".format(
-                info["bearer"]["service"], repo
-            ),
-            auth=normalized_auth_info,  # type: ignore
-            timeout=3,
-        )
-        res.raise_for_status()
-        result_json: Dict[str, str] = res.json()
-        return result_json
-    return {}
-
-
 def image_id_from_registry(image_name: str) -> Optional[str]:
-    """Get the docker id from a public or private registry."""
-    registry, repository, tag = parse(image_name)
-    res = None
-    try:
-        token = auth_token(registry, repository).get("token")
-        # dockerhub is crazy
-        if registry == "index.docker.io":
-            registry = "registry-1.docker.io"
-        res = requests.head(
-            f"https://{registry}/v2/{repository}/manifests/{tag}",
-            headers={
-                "Authorization": f"Bearer {token}",
-                "Accept": "application/vnd.docker.distribution.manifest.v2+json",
-            },
-            timeout=5,
-        )
-        res.raise_for_status()
-    except requests.RequestException:
-        log.error(f"Received {res} when attempting to get digest for {image_name}")
-        return None
-    return "@".join([registry + "/" + repository, res.headers["Docker-Content-Digest"]])
+    """Query the image manifest to get its full ID including the digest.
+
+    Args:
+        image_name: The image name, such as "wandb/local".
+
+    Returns:
+        The image name followed by its digest, like "wandb/local@sha256:...".
+    """
+    # https://docs.docker.com/reference/cli/docker/buildx/imagetools/inspect
+    inspect_cmd = ["buildx", "imagetools", "inspect", image_name]
+    format_args = ["--format", r"{{.Name}}@{{.Manifest.Digest}}"]
+    return shell([*inspect_cmd, *format_args])
 
 
 def image_id(image_name: str) -> Optional[str]:
-    """Retreve the image id from the local docker daemon or remote registry."""
+    """Retrieve the image id from the local docker daemon or remote registry."""
     if "@sha256:" in image_name:
         return image_name
     else:
         digests = shell(["inspect", image_name, "--format", "{{json .RepoDigests}}"])
+
+        if digests is None:
+            return image_id_from_registry(image_name)
+
         try:
-            if digests is None:
-                raise ValueError
-            im_id: str = json.loads(digests)[0]
-            return im_id
+            return json.loads(digests)[0]
         except (ValueError, IndexError):
             return image_id_from_registry(image_name)
 
@@ -302,7 +279,7 @@ __all__ = [
     "run",
     "image_id",
     "image_id_from_registry",
-    "auth_token",
+    "is_docker_installed",
     "parse",
     "parse_repository_tag",
     "default_image",
