@@ -1,6 +1,4 @@
-"""
-keras init
-"""
+"""keras init."""
 
 import logging
 import operator
@@ -11,31 +9,30 @@ from itertools import chain
 
 import numpy as np
 import tensorflow as tf
-import tensorflow.keras.backend as K
+import tensorflow.keras.backend as K  # noqa: N812
 
 import wandb
+from wandb.proto.wandb_deprecated import Deprecated
 from wandb.sdk.integration_utils.data_logging import ValidationDataLogger
-from wandb.sdk.lib.deprecate import Deprecated, deprecate
+from wandb.sdk.lib.deprecate import deprecate
 from wandb.util import add_import_hook
 
 
 def _check_keras_version():
     from keras import __version__ as keras_version
-    from pkg_resources import parse_version
+    from packaging.version import parse
 
-    if parse_version(keras_version) < parse_version("2.4.0"):
+    if parse(keras_version) < parse("2.4.0"):
         wandb.termwarn(
             f"Keras version {keras_version} is not fully supported. Required keras >= 2.4.0"
         )
 
 
 def _can_compute_flops() -> bool:
-    """
-    FLOPS computation is restricted to TF 2.x as it requires tf.compat.v1
-    """
-    from pkg_resources import parse_version
+    """FLOPS computation is restricted to TF 2.x as it requires tf.compat.v1."""
+    from packaging.version import parse
 
-    if parse_version(tf.__version__) >= parse_version("2.0.0"):
+    if parse(tf.__version__) >= parse("2.0.0"):
         return True
 
     return False
@@ -76,11 +73,11 @@ def is_generator_like(data):
     return hasattr(data, "next") or hasattr(data, "__next__") or isinstance(data, types)
 
 
-def patch_tf_keras():
-    from pkg_resources import parse_version
+def patch_tf_keras():  # noqa: C901
+    from packaging.version import parse
     from tensorflow.python.eager import context
 
-    if parse_version(tf.__version__) >= parse_version("2.6.0"):
+    if parse("2.6.0") <= parse(tf.__version__) < parse("2.13.0"):
         keras_engine = "keras.engine"
         try:
             from keras.engine import training
@@ -235,9 +232,9 @@ patch_tf_keras()
 
 
 def _get_custom_optimizer_parent_class():
-    from pkg_resources import parse_version
+    from packaging.version import parse
 
-    if parse_version(tf.__version__) >= parse_version("2.9.0"):
+    if parse(tf.__version__) >= parse("2.9.0"):
         custom_optimizer_parent_class = tf.keras.optimizers.legacy.Optimizer
     else:
         custom_optimizer_parent_class = tf.keras.optimizers.Optimizer
@@ -267,10 +264,7 @@ class _CustomOptimizer(_custom_optimizer_parent_class):
 
 
 class _GradAccumulatorCallback(tf.keras.callbacks.Callback):
-    """
-    Accumulates gradients during a fit() call when used in conjunction with
-    the CustomOptimizer above.
-    """
+    """Accumulates gradients during a fit() call when used in conjunction with the CustomOptimizer above."""
 
     def set_model(self, model):
         super().set_model(model)
@@ -314,7 +308,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
 
     `WandbCallback` can optionally save training and validation data for wandb to visualize.
 
-    Arguments:
+    Args:
         monitor: (str) name of metric to monitor.  Defaults to `val_loss`.
         mode: (str) one of {`auto`, `min`, `max`}.
             `min` - save model when monitor is minimized
@@ -397,7 +391,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
         save_model=True,
         training_data=None,
         validation_data=None,
-        labels=[],
+        labels=None,
         predictions=36,
         generator=None,
         input_type=None,
@@ -418,6 +412,17 @@ class WandbCallback(tf.keras.callbacks.Callback):
     ):
         if wandb.run is None:
             raise wandb.Error("You must call wandb.init() before WandbCallback()")
+
+        deprecate(
+            field_name=Deprecated.keras_callback,
+            warning_message=(
+                "WandbCallback is deprecated and will be removed in a future release. "
+                "Please use the WandbMetricsLogger, WandbModelCheckpoint, and WandbEvalCallback "
+                "callbacks instead. "
+                "See https://docs.wandb.ai/guides/integrations/keras for more information."
+            ),
+        )
+
         with wandb.wandb_lib.telemetry.context(run=wandb.run) as tel:
             tel.feature.keras = True
         self.validation_data = None
@@ -427,7 +432,8 @@ class WandbCallback(tf.keras.callbacks.Callback):
                 generator = validation_data
             else:
                 self.validation_data = validation_data
-
+        if labels is None:
+            labels = []
         self.labels = labels
         self.predictions = min(predictions, 100)
 
@@ -498,7 +504,9 @@ class WandbCallback(tf.keras.callbacks.Callback):
 
         # From Keras
         if mode not in ["auto", "min", "max"]:
-            print(f"WandbCallback mode {mode} is unknown, fallback to auto mode.")
+            wandb.termwarn(
+                f"WandbCallback mode {mode} is unknown, fallback to auto mode."
+            )
             mode = "auto"
 
         if mode == "min":
@@ -552,7 +560,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
         self.params = params
 
     def set_model(self, model):
-        self.model = model
+        super().set_model(model)
         if self.input_type == "auto" and len(model.inputs) == 1:
             self.input_type = wandb.util.guess_data_type(
                 model.inputs[0].shape, risky=True
@@ -576,9 +584,11 @@ class WandbCallback(tf.keras.callbacks.Callback):
                     )
                     self._model_trained_since_last_eval = False
             except Exception as e:
-                wandb.termwarn("Error durring prediction logging for epoch: " + str(e))
+                wandb.termwarn("Error during prediction logging for epoch: " + str(e))
 
-    def on_epoch_end(self, epoch, logs={}):
+    def on_epoch_end(self, epoch, logs=None):
+        if logs is None:
+            logs = {}
         if self.log_weights:
             wandb.log(self._log_weights(), commit=False)
 
@@ -614,14 +624,13 @@ class WandbCallback(tf.keras.callbacks.Callback):
         self.current = logs.get(self.monitor)
         if self.current and self.monitor_op(self.current, self.best):
             if self.log_best_prefix:
-                wandb.run.summary[
-                    f"{self.log_best_prefix}{self.monitor}"
-                ] = self.current
+                wandb.run.summary[f"{self.log_best_prefix}{self.monitor}"] = (
+                    self.current
+                )
                 wandb.run.summary["{}{}".format(self.log_best_prefix, "epoch")] = epoch
                 if self.verbose and not self.save_model:
-                    print(
-                        "Epoch %05d: %s improved from %0.5f to %0.5f"
-                        % (epoch, self.monitor, self.best, self.current)
+                    wandb.termlog(
+                        f"Epoch {epoch:05d}: {self.monitor} improved from {self.best:.5f} to {self.current:.5f}"
                     )
             if self.save_model:
                 self._save_model(epoch)
@@ -684,7 +693,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
                     else:
                         x = None
                         y_true = None
-                        for i in range(self.validation_steps):
+                        for _ in range(self.validation_steps):
                             bx, by_true = next(self.generator)
                             if x is None:
                                 x, y_true = bx, by_true
@@ -719,25 +728,14 @@ class WandbCallback(tf.keras.callbacks.Callback):
         if self.compute_flops and _can_compute_flops():
             try:
                 wandb.summary["GFLOPs"] = self.get_flops()
-            except Exception as e:
+            except Exception:
+                logger.exception("Error computing FLOPs")
                 wandb.termwarn("Unable to compute FLOPs for this model.")
 
     def on_train_end(self, logs=None):
         if self._model_trained_since_last_eval:
             self._attempt_evaluation_log()
 
-    def on_test_begin(self, logs=None):
-        pass
-
-    def on_test_end(self, logs=None):
-        pass
-
-    def on_test_batch_begin(self, batch, logs=None):
-        pass
-
-    def on_test_batch_end(self, batch, logs=None):
-        pass
-
     def on_predict_begin(self, logs=None):
         pass
 
@@ -797,7 +795,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
         return imgs
 
     def _log_images(self, num_images=36):
-        validation_X = self.validation_data[0]
+        validation_X = self.validation_data[0]  # noqa: N806
         validation_y = self.validation_data[1]
 
         validation_length = len(validation_X)
@@ -945,10 +943,10 @@ class WandbCallback(tf.keras.callbacks.Callback):
         weights = self.model.trainable_weights
         grads = self._grad_accumulator_callback.grads
         metrics = {}
-        for (weight, grad) in zip(weights, grads):
-            metrics[
-                "gradients/" + weight.name.split(":")[0] + ".gradient"
-            ] = wandb.Histogram(grad)
+        for weight, grad in zip(weights, grads):
+            metrics["gradients/" + weight.name.split(":")[0] + ".gradient"] = (
+                wandb.Histogram(grad)
+            )
         return metrics
 
     def _log_dataframe(self):
@@ -965,7 +963,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
                 )
                 return None
 
-            for i in range(self.validation_steps):
+            for _ in range(self.validation_steps):
                 bx, by_true = next(self.generator)
                 by_pred = self.model.predict(bx)
                 if x is None:
@@ -994,8 +992,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
             )
         else:
             wandb.termwarn(
-                "unknown dataframe type for input_type=%s and output_type=%s"
-                % (self.input_type, self.output_type)
+                f"unknown dataframe type for input_type={self.input_type} and output_type={self.output_type}"
             )
             return None
 
@@ -1003,10 +1000,9 @@ class WandbCallback(tf.keras.callbacks.Callback):
         if wandb.run.disabled:
             return
         if self.verbose > 0:
-            print(
-                "Epoch %05d: %s improved from %0.5f to %0.5f,"
-                " saving model to %s"
-                % (epoch, self.monitor, self.best, self.current, self.filepath)
+            wandb.termlog(
+                f"Epoch {epoch:05d}: {self.monitor} improved from {self.best:.5f} to {self.current:.5f}, "
+                f"saving model to {self.filepath}"
             )
 
         try:
@@ -1016,7 +1012,8 @@ class WandbCallback(tf.keras.callbacks.Callback):
                 self.model.save(self.filepath, overwrite=True)
         # Was getting `RuntimeError: Unable to create link` in TF 1.13.1
         # also saw `TypeError: can't pickle _thread.RLock objects`
-        except (ImportError, RuntimeError, TypeError, AttributeError) as e:
+        except (ImportError, RuntimeError, TypeError, AttributeError):
+            logger.exception("Error saving model in the h5py format")
             wandb.termerror(
                 "Can't save model in the h5py format. The model will be saved as "
                 "as an W&B Artifact in the 'tf' format."
@@ -1041,9 +1038,9 @@ class WandbCallback(tf.keras.callbacks.Callback):
         shutil.rmtree(self.filepath[:-3])
 
     def get_flops(self) -> float:
-        """
-        Calculate FLOPS [GFLOPs] for a tf.keras.Model or tf.keras.Sequential model
-        in inference mode. It uses tf.compat.v1.profiler under the hood.
+        """Calculate FLOPS [GFLOPs] for a tf.keras.Model or tf.keras.Sequential model in inference mode.
+
+        It uses tf.compat.v1.profiler under the hood.
         """
         if not hasattr(self, "model"):
             raise wandb.Error("self.model must be set before using this method.")
@@ -1051,7 +1048,7 @@ class WandbCallback(tf.keras.callbacks.Callback):
         if not isinstance(
             self.model, (tf.keras.models.Sequential, tf.keras.models.Model)
         ):
-            raise ValueError(
+            raise TypeError(
                 "Calculating FLOPS is only supported for "
                 "`tf.keras.Model` and `tf.keras.Sequential` instances."
             )
