@@ -1,4 +1,4 @@
-"""W&B callback for sb3
+"""W&B callback for sb3.
 
 Really simple callback to get logging for each tree
 
@@ -34,7 +34,9 @@ def make_env():
 
 
 env = DummyVecEnv([make_env])
-env = VecVideoRecorder(env, "videos", record_video_trigger=lambda x: x % 2000 == 0, video_length=200)
+env = VecVideoRecorder(
+    env, "videos", record_video_trigger=lambda x: x % 2000 == 0, video_length=200
+)
 model = PPO(config["policy_type"], env, verbose=1, tensorboard_log=f"runs")
 model.learn(
     total_timesteps=config["total_timesteps"],
@@ -49,13 +51,7 @@ model.learn(
 
 import logging
 import os
-import sys
-from typing import Optional
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+from typing import Literal, Optional
 
 from stable_baselines3.common.callbacks import BaseCallback  # type: ignore
 
@@ -66,11 +62,13 @@ logger = logging.getLogger(__name__)
 
 
 class WandbCallback(BaseCallback):
-    """Log SB3 experiments to Weights and Biases
+    """Callback for logging experiments to Weights and Biases.
+
+    Log SB3 experiments to Weights and Biases
         - Added model tracking and uploading
         - Added complete hyperparameters recording
         - Added gradient logging
-        - Note that `wandb.init(...)` must be called before the WandbCallback can be used
+        - Note that `wandb.init(...)` must be called before the WandbCallback can be used.
 
     Args:
         verbose: The verbosity of sb3 output
@@ -108,9 +106,9 @@ class WandbCallback(BaseCallback):
             os.makedirs(self.model_save_path, exist_ok=True)
             self.path = os.path.join(self.model_save_path, "model.zip")
         else:
-            assert (
-                self.model_save_freq == 0
-            ), "to use the `model_save_freq` you have to set the `model_save_path` parameter"
+            assert self.model_save_freq == 0, (
+                "to use the `model_save_freq` you have to set the `model_save_path` parameter"
+            )
 
     def _init_callback(self) -> None:
         d = {}
