@@ -1,40 +1,40 @@
 import codecs
+import itertools
 import json
 import os
-import sys
+import pathlib
 from typing import (
     TYPE_CHECKING,
     ClassVar,
+    Literal,
     Optional,
     Sequence,
     Set,
     TextIO,
     Tuple,
     Type,
+    TypedDict,
     Union,
 )
 
-if sys.version_info >= (3, 8):
-    from typing import Literal, TypedDict
-else:
-    from typing_extensions import Literal, TypedDict
-
-
 import wandb
 from wandb import util
 from wandb.sdk.lib import runid
+from wandb.sdk.lib.paths import LogicalPath
 
 from . import _dtypes
 from ._private import MEDIA_TMP
 from .base_types.media import BatchableMedia
 
 if TYPE_CHECKING:  # pragma: no cover
-    import numpy as np  # type: ignore
+    import numpy as np
+    import numpy.typing as npt
+
+    from wandb.sdk.artifacts.artifact import Artifact
 
-    from ..wandb_artifacts import Artifact as LocalArtifact
     from ..wandb_run import Run as LocalRun
 
-    numeric = Union[int, float, np.integer, np.float_]
+    numeric = Union[int, float, np.integer, np.float64]
     FileFormat3D = Literal[
         "obj",
         "gltf",
@@ -48,7 +48,9 @@ if TYPE_CHECKING:  # pragma: no cover
     Point3DWithColors = Tuple[numeric, numeric, numeric, numeric, numeric, numeric]
     Point = Union[Point3D, Point3DWithCategory, Point3DWithColors]
     PointCloudType = Literal["lidar/beta"]
-    RGBColor = Tuple[int, int, int]
+    RGBColor = Tuple[numeric, numeric, numeric]
+
+    Quaternion = Tuple[numeric, numeric, numeric, numeric]
 
     class Box3D(TypedDict):
         corners: Tuple[
@@ -74,23 +76,116 @@ if TYPE_CHECKING:  # pragma: no cover
         target: Sequence[Point3D]
 
 
-class Object3D(BatchableMedia):
-    """Wandb class for 3D point clouds.
+def _install_numpy_error() -> "wandb.Error":
+    return wandb.Error(
+        "wandb.Object3D requires NumPy. To get it, run 'pip install numpy'."
+    )
 
-    Arguments:
-        data_or_path: (numpy array, string, io)
-            Object3D can be initialized from a file or a numpy array.
 
-            You can pass a path to a file or an io object and a file_type
-            which must be one of SUPPORTED_TYPES
+def _normalize_vec(x: "np.ndarray") -> "np.ndarray":
+    """Normalizes a non-zero 1-dimensional array."""
+    try:
+        import numpy as np
+    except ImportError as e:
+        raise _install_numpy_error() from e
 
-    The shape of the numpy array must be one of either:
-    ```
-    [[x y z],       ...] nx3
-    [[x y z c],     ...] nx4 where c is a category with supported range [1, 14]
-    [[x y z r g b], ...] nx6 where is rgb is color
-    ```
+    x = abs(x)
+    argmax = x.argmax()
+
+    if x[argmax] == 0:
+        raise ValueError("Unexpected zero quaternion.")
+
+    # Rescale by the largest component first to be more numerically
+    # stable than dividing by the norm directly.
+    x = x / x[argmax]
+    return x / np.linalg.norm(x)
+
+
+def _quaternion_to_rotation(quaternion: "np.ndarray") -> "np.ndarray":
+    """Returns the rotation matrix corresponding to a non-zero quaternion.
+
+    The corresponding rotation matrix transforms column vectors by
+    post-multiplication: `x @ R`. This way, it can be used to
+    transform an Nx3 NumPy array of points as `points @ R`.
     """
+    try:
+        import numpy as np
+    except ImportError as e:
+        raise _install_numpy_error() from e
+
+    # Precompute a few products to simplify the expression below.
+    qr, qi, qj, qk = _normalize_vec(quaternion)
+    qii, qjj, qkk = qi**2, qj**2, qk**2
+    qij, qik, qjk = qi * qj, qi * qk, qj * qk
+    qir, qjr, qkr = qi * qr, qj * qr, qk * qr
+
+    return np.array(
+        (
+            (1 - 2 * (qjj + qkk), 2 * (qij + qkr), 2 * (qik - qjr)),
+            (2 * (qij - qkr), 1 - 2 * (qii + qkk), 2 * (qjk + qir)),
+            (2 * (qik + qjr), 2 * (qjk - qir), 1 - 2 * (qii + qjj)),
+        ),
+        dtype=np.float64,
+    )
+
+
+def box3d(
+    *,
+    center: "npt.ArrayLike",
+    size: "npt.ArrayLike",
+    orientation: "npt.ArrayLike",
+    color: "RGBColor",
+    label: "Optional[str]" = None,
+    score: "Optional[numeric]" = None,
+) -> "Box3D":
+    """Returns a Box3D.
+
+    Args:
+        center: The center point of the box as a length-3 ndarray.
+        size: The box's X, Y and Z dimensions as a length-3 ndarray.
+        orientation: The rotation transforming global XYZ coordinates
+            into the box's local XYZ coordinates, given as a length-4
+            ndarray [r, x, y, z] corresponding to the non-zero quaternion
+            r + xi + yj + zk.
+        color: The box's color as an (r, g, b) tuple with 0 <= r,g,b <= 1.
+        label: An optional label for the box.
+        score: An optional score for the box.
+    """
+    try:
+        import numpy as np
+    except ImportError as e:
+        raise _install_numpy_error() from e
+
+    center = np.asarray(center, dtype=np.float64)
+    size = np.asarray(size, dtype=np.float64)
+    orientation = np.asarray(orientation, dtype=np.float64)
+
+    assert center.shape == (3,)
+    assert size.shape == (3,)
+    assert orientation.shape == (4,)
+
+    # Precompute the rotation matrix.
+    rot = _quaternion_to_rotation(orientation)
+
+    # Scale, rotate and translate each corner of the unit box.
+    unit_corners = np.array(
+        list(itertools.product((-1, 1), (-1, 1), (-1, 1))),
+        dtype=np.float64,
+    )
+    corners = center + (0.5 * size * unit_corners) @ rot
+
+    return {
+        # Ignore the type because mypy can't infer that the list has length 8:
+        # https://github.com/python/mypy/issues/7509
+        "corners": tuple(tuple(pt) for pt in corners),  # type: ignore
+        "color": color,
+        "label": label,
+        "score": score,
+    }
+
+
+class Object3D(BatchableMedia):
+    """W&B class for 3D point clouds."""
 
     SUPPORTED_TYPES: ClassVar[Set[str]] = {
         "obj",
@@ -105,13 +200,31 @@ class Object3D(BatchableMedia):
 
     def __init__(
         self,
-        data_or_path: Union["np.ndarray", str, "TextIO", dict],
+        data_or_path: Union["np.ndarray", str, pathlib.Path, "TextIO", dict],
+        caption: Optional[str] = None,
         **kwargs: Optional[Union[str, "FileFormat3D"]],
     ) -> None:
-        super().__init__()
+        """Creates a W&B Object3D object.
 
-        if hasattr(data_or_path, "name"):
-            # if the file has a path, we just detect the type and copy it from there
+        Args:
+            data_or_path: Object3D can be initialized from a file or a numpy array.
+            caption: Caption associated with the object for display.
+
+        Examples:
+        The shape of the numpy array must be one of either
+
+        ```text
+        [[x y z],       ...] nx3
+        [[x y z c],     ...] nx4 where c is a category with supported range [1, 14]
+        [[x y z r g b], ...] nx6 where is rgb is color
+        ```
+        """
+        super().__init__(caption=caption)
+
+        if hasattr(data_or_path, "name") and not isinstance(data_or_path, pathlib.Path):
+            # if the file has a path, we just detect the type and copy it from there.
+            # this does not work for pathlib.Path objects,
+            # where `.name` returns the last directory in the path.
             data_or_path = data_or_path.name
 
         if hasattr(data_or_path, "read"):
@@ -137,7 +250,9 @@ class Object3D(BatchableMedia):
                 f.write(object_3d)
 
             self._set_file(tmp_path, is_tmp=True, extension=extension)
-        elif isinstance(data_or_path, str):
+        elif isinstance(data_or_path, (str, pathlib.Path)):
+            data_or_path = str(data_or_path)
+
             path = data_or_path
             extension = None
             for supported_type in Object3D.SUPPORTED_TYPES:
@@ -228,10 +343,12 @@ class Object3D(BatchableMedia):
     ) -> "Object3D":
         """Initializes Object3D from a file or stream.
 
-        Arguments:
+        Args:
             data_or_path (Union["TextIO", str]): A path to a file or a `TextIO` stream.
             file_type (str): Specifies the data format passed to `data_or_path`. Required when `data_or_path` is a
                 `TextIO` stream. This parameter is ignored if a file path is provided. The type is taken from the file extension.
+
+        <!-- lazydoc-ignore-classmethod: internal -->
         """
         # if file_type is not None and file_type not in cls.SUPPORTED_TYPES:
         #     raise ValueError(
@@ -243,17 +360,20 @@ class Object3D(BatchableMedia):
     def from_numpy(cls, data: "np.ndarray") -> "Object3D":
         """Initializes Object3D from a numpy array.
 
-        Arguments:
+        Args:
             data (numpy array): Each entry in the array will
                 represent one point in the point cloud.
 
 
         The shape of the numpy array must be one of either:
-        ```
+
+        ```text
         [[x y z],       ...]  # nx3.
         [[x y z c],     ...]  # nx4 where c is a category with supported range [1, 14].
         [[x y z r g b], ...]  # nx6 where is rgb is color.
         ```
+
+        <!-- lazydoc-ignore-classmethod: internal -->
         """
         if not util.is_numpy_array(data):
             raise ValueError("`data` must be a numpy array")
@@ -281,13 +401,15 @@ class Object3D(BatchableMedia):
     ) -> "Object3D":
         """Initializes Object3D from a python object.
 
-        Arguments:
+        Args:
             points (Sequence["Point"]): The points in the point cloud.
             boxes (Sequence["Box3D"]): 3D bounding boxes for labeling the point cloud. Boxes
             are displayed in point cloud visualizations.
             vectors (Optional[Sequence["Vector3D"]]): Each vector is displayed in the point cloud
                 visualization. Can be used to indicate directionality of bounding boxes. Defaults to None.
             point_cloud_type ("lidar/beta"): At this time, only the "lidar/beta" type is supported. Defaults to "lidar/beta".
+
+        <!-- lazydoc-ignore-classmethod: internal -->
         """
         if point_cloud_type not in cls.SUPPORTED_POINT_CLOUD_TYPES:
             raise ValueError("Point cloud type not supported")
@@ -308,13 +430,21 @@ class Object3D(BatchableMedia):
 
     @classmethod
     def get_media_subdir(cls: Type["Object3D"]) -> str:
+        """Get media subdirectory.
+
+        <!-- lazydoc-ignore-classmethod: internal -->
+        """
         return os.path.join("media", "object3D")
 
-    def to_json(self, run_or_artifact: Union["LocalRun", "LocalArtifact"]) -> dict:
+    def to_json(self, run_or_artifact: Union["LocalRun", "Artifact"]) -> dict:
+        """Returns the JSON representation expected by the backend.
+
+        <!-- lazydoc-ignore: internal -->
+        """
         json_dict = super().to_json(run_or_artifact)
         json_dict["_type"] = Object3D._log_type
 
-        if isinstance(run_or_artifact, wandb.wandb_sdk.wandb_artifacts.Artifact):
+        if isinstance(run_or_artifact, wandb.Artifact):
             if self._path is None or not self._path.endswith(".pts.json"):
                 raise ValueError(
                     "Non-point cloud 3D objects are not yet supported with Artifacts"
@@ -330,12 +460,16 @@ class Object3D(BatchableMedia):
         key: str,
         step: Union[int, str],
     ) -> dict:
+        """Convert a sequence of Audio objects to a JSON representation.
+
+        <!-- lazydoc-ignore-classmethod: internal -->
+        """
         seq = list(seq)
 
         jsons = [obj.to_json(run) for obj in seq]
 
         for obj in jsons:
-            expected = util.to_forward_slash_path(cls.get_media_subdir())
+            expected = LogicalPath(cls.get_media_subdir())
             if not obj["path"].startswith(expected):
                 raise ValueError(
                     "Files in an array of Object3D's must be in the {} directory, not {}".format(
