@@ -3,14 +3,10 @@ from typing import Any, Callable, Dict
 
 log = logging.getLogger(__name__)
 
-SCHEDULER_URI = "placeholder-uri-scheduler"
-
 
 class SchedulerError(Exception):
     """Raised when a known error occurs with wandb sweep scheduler."""
 
-    pass
-
 
 def _import_sweep_scheduler() -> Any:
     from .scheduler_sweep import SweepScheduler
@@ -19,20 +15,20 @@ def _import_sweep_scheduler() -> Any:
 
 
 _WANDB_SCHEDULERS: Dict[str, Callable] = {
-    "sweep": _import_sweep_scheduler,
+    "wandb": _import_sweep_scheduler,
 }
 
 
-def load_scheduler(scheduler_name: str) -> Any:
-    scheduler_name = scheduler_name.lower()
-    if scheduler_name not in _WANDB_SCHEDULERS:
+def load_scheduler(scheduler_type: str) -> Any:
+    scheduler_type = scheduler_type.lower()
+    if scheduler_type not in _WANDB_SCHEDULERS:
         raise SchedulerError(
             f"The `scheduler_name` argument must be one of "
-            f"{list(_WANDB_SCHEDULERS.keys())}, got: {scheduler_name}"
+            f"{list(_WANDB_SCHEDULERS.keys())}, got: {scheduler_type}"
         )
 
-    log.warn(f"Loading dependencies for Scheduler of type: {scheduler_name}")
-    import_func = _WANDB_SCHEDULERS[scheduler_name]
+    log.warning(f"Loading dependencies for Scheduler of type: {scheduler_type}")
+    import_func = _WANDB_SCHEDULERS[scheduler_type]
     return import_func()
 
 
