@@ -1,5 +1,8 @@
 """Utilities for wandb verify."""
+
+import contextlib
 import getpass
+import io
 import os
 import time
 from functools import partial
@@ -8,21 +11,19 @@ from typing import Any, Callable, Dict, List, Optional, Tuple, Union
 
 import click
 import requests
-from pkg_resources import parse_version
 from wandb_gql import gql
 
 import wandb
+from wandb.sdk.artifacts.artifact import Artifact
 from wandb.sdk.lib import runid
 
 from ...apis.internal import Api
-from ...apis.public import Artifact as ArtifactAPI
-from ..wandb_artifacts import Artifact
 
 PROJECT_NAME = "verify"
 GET_RUN_MAX_TIME = 10
 MIN_RETRYS = 3
 CHECKMARK = "\u2705"
-RED_X = "\u274C"
+RED_X = "\u274c"
 ID_PREFIX = runid.generate_id()
 
 
@@ -38,17 +39,17 @@ def print_results(
     else:
         color = "red"
     if isinstance(failed_test_or_tests, str):
-        print(RED_X)
-        print(click.style(failed_test_or_tests, fg=color, bold=True))
+        print(RED_X)  # noqa: T201
+        print(click.style(failed_test_or_tests, fg=color, bold=True))  # noqa: T201
     elif isinstance(failed_test_or_tests, list) and len(failed_test_or_tests) > 0:
-        print(RED_X)
-        print(
+        print(RED_X)  # noqa: T201
+        print(  # noqa: T201
             "\n".join(
                 [click.style(f, fg=color, bold=True) for f in failed_test_or_tests]
             )
         )
     else:
-        print(CHECKMARK)
+        print(CHECKMARK)  # noqa: T201
 
 
 def check_host(host: str) -> bool:
@@ -59,7 +60,7 @@ def check_host(host: str) -> bool:
 
 
 def check_logged_in(api: Api, host: str) -> bool:
-    print("Checking if logged in".ljust(72, "."), end="")
+    print("Checking if logged in".ljust(72, "."), end="")  # noqa: T201
     login_doc_url = "https://docs.wandb.ai/ref/cli/wandb-login"
     fail_string = None
     if api.api_key is None:
@@ -84,7 +85,7 @@ def check_logged_in(api: Api, host: str) -> bool:
 
 def check_secure_requests(url: str, test_url_string: str, failure_output: str) -> None:
     # check if request is over https
-    print(test_url_string.ljust(72, "."), end="")
+    print(test_url_string.ljust(72, "."), end="")  # noqa: T201
     fail_string = None
     if not url.startswith("https"):
         fail_string = failure_output
@@ -92,7 +93,7 @@ def check_secure_requests(url: str, test_url_string: str, failure_output: str) -
 
 
 def check_cors_configuration(url: str, origin: str) -> None:
-    print("Checking CORs configuration of the bucket".ljust(72, "."), end="")
+    print("Checking CORs configuration of the bucket".ljust(72, "."), end="")  # noqa: T201
     fail_string = None
     res_get = requests.options(
         url, headers={"Origin": origin, "Access-Control-Request-Method": "GET"}
@@ -108,7 +109,7 @@ def check_cors_configuration(url: str, origin: str) -> None:
 
 
 def check_run(api: Api) -> bool:
-    print(
+    print(  # noqa: T201
         "Checking logged metrics, saving and downloading a file".ljust(72, "."), end=""
     )
     failed_test_strings = []
@@ -131,7 +132,10 @@ def check_run(api: Api) -> bool:
     f.close()
 
     with wandb.init(
-        id=nice_id("check_run"), reinit=True, config=config, project=PROJECT_NAME
+        id=nice_id("check_run"),
+        reinit=True,
+        config=config,
+        project=PROJECT_NAME,
     ) as run:
         run_id = run.id
         entity = run.entity
@@ -154,7 +158,7 @@ def check_run(api: Api) -> bool:
                 "Failed to log to media. Contact W&B for support."
             )
 
-        wandb.save(filepath)
+        run.save(filepath)
     public_api = wandb.Api()
     prev_run = public_api.run(f"{entity}/{PROJECT_NAME}/{run_id}")
     # raise Exception(prev_run.__dict__)
@@ -164,8 +168,8 @@ def check_run(api: Api) -> bool:
         )
         print_results(failed_test_strings, False)
         return False
-    for key, value in prev_run.config.items():
-        if config[key] != value:
+    for key, value in config.items():
+        if prev_run.config.get(key) != value:
             failed_test_strings.append(
                 "Read config values don't match run config. Contact W&B for support."
             )
@@ -224,7 +228,7 @@ def verify_manifest(
 
 
 def verify_digest(
-    downloaded: "ArtifactAPI", computed: "ArtifactAPI", fails_list: List[str]
+    downloaded: "Artifact", computed: "Artifact", fails_list: List[str]
 ) -> None:
     if downloaded.digest != computed.digest:
         fails_list.append(
@@ -268,7 +272,7 @@ def log_use_download_artifact(
     download_dir: str,
     failed_test_strings: List[str],
     add_extra_file: bool,
-) -> Tuple[bool, Optional["ArtifactAPI"], List[str]]:
+) -> Tuple[bool, Optional["Artifact"], List[str]]:
     with wandb.init(
         id=nice_id("log_artifact"),
         reinit=True,
@@ -309,13 +313,13 @@ def log_use_download_artifact(
 
 
 def check_artifacts() -> bool:
-    print("Checking artifact save and download workflows".ljust(72, "."), end="")
+    print("Checking artifact save and download workflows".ljust(72, "."), end="")  # noqa: T201
     failed_test_strings: List[str] = []
 
     # test checksum
     sing_art_dir = "./verify_sing_art"
     alias = "sing_art1"
-    name = "sing-artys"
+    name = nice_id("sing-artys")
     singular_art = artifact_with_path_or_paths(name, singular=True)
     cont_test, download_artifact, failed_test_strings = log_use_download_artifact(
         singular_art, alias, name, sing_art_dir, failed_test_strings, False
@@ -333,7 +337,7 @@ def check_artifacts() -> bool:
     # test manifest and digest
     multi_art_dir = "./verify_art"
     alias = "art1"
-    name = "my-artys"
+    name = nice_id("my-artys")
     art1 = artifact_with_path_or_paths(name, "./verify_art_dir", singular=False)
     cont_test, download_artifact, failed_test_strings = log_use_download_artifact(
         art1, alias, name, multi_art_dir, failed_test_strings, True
@@ -357,9 +361,7 @@ def check_artifacts() -> bool:
     verify_digest(download_artifact, computed, failed_test_strings)
 
     computed_manifest = computed.manifest.to_manifest_json()["contents"]
-    downloaded_manifest = download_artifact._load_manifest().to_manifest_json()[
-        "contents"
-    ]
+    downloaded_manifest = download_artifact.manifest.to_manifest_json()["contents"]
     verify_manifest(downloaded_manifest, computed_manifest, failed_test_strings)
 
     print_results(failed_test_strings, False)
@@ -368,7 +370,7 @@ def check_artifacts() -> bool:
 
 def check_graphql_put(api: Api, host: str) -> Tuple[bool, Optional[str]]:
     # check graphql endpoint using an upload
-    print("Checking signed URL upload".ljust(72, "."), end="")
+    print("Checking signed URL upload".ljust(72, "."), end="")  # noqa: T201
     failed_test_strings = []
     gql_fp = "gql_test_file.txt"
     f = open(gql_fp, "w")
@@ -380,7 +382,7 @@ def check_graphql_put(api: Api, host: str) -> Tuple[bool, Optional[str]]:
         project=PROJECT_NAME,
         config={"test": "put to graphql"},
     ) as run:
-        wandb.save(gql_fp)
+        run.save(gql_fp)
     public_api = wandb.Api()
     prev_run = public_api.run(f"{run.entity}/{PROJECT_NAME}/{run.id}")
     if prev_run is None:
@@ -413,7 +415,7 @@ def check_graphql_put(api: Api, host: str) -> Tuple[bool, Optional[str]]:
 
 
 def check_large_post() -> bool:
-    print(
+    print(  # noqa: T201
         "Checking ability to send large payloads through proxy".ljust(72, "."), end=""
     )
     descy = "a" * int(10**7)
@@ -447,7 +449,11 @@ def check_large_post() -> bool:
             timeout=60,
         )
     except Exception as e:
-        if isinstance(e, requests.HTTPError) and e.response.status_code == 413:
+        if (
+            isinstance(e, requests.HTTPError)
+            and e.response is not None
+            and e.response.status_code == 413
+        ):
             failed_test_strings.append(
                 'Failed to send a large payload. Check nginx.ingress.kubernetes.io/proxy-body-size is "0".'
             )
@@ -460,7 +466,7 @@ def check_large_post() -> bool:
 
 
 def check_wandb_version(api: Api) -> None:
-    print("Checking wandb package version is up to date".ljust(72, "."), end="")
+    print("Checking wandb package version is up to date".ljust(72, "."), end="")  # noqa: T201
     _, server_info = api.viewer_server_info()
     fail_string = None
     warning = False
@@ -468,11 +474,12 @@ def check_wandb_version(api: Api) -> None:
     min_cli_version = server_info.get("cliVersionInfo", {}).get(
         "min_cli_version", "0.0.1"
     )
-    if parse_version(wandb.__version__) < parse_version(min_cli_version):
-        fail_string = "wandb version out of date, please run pip install --upgrade wandb=={}".format(
-            max_cli_version
-        )
-    elif parse_version(wandb.__version__) > parse_version(max_cli_version):
+
+    from packaging.version import parse
+
+    if parse(wandb.__version__) < parse(min_cli_version):
+        fail_string = f"wandb version out of date, please run pip install --upgrade wandb=={max_cli_version}"
+    elif parse(wandb.__version__) > parse(max_cli_version):
         fail_string = (
             "wandb version is not supported by your local installation. This could "
             "cause some issues. If you're having problems try: please run `pip "
@@ -483,6 +490,56 @@ def check_wandb_version(api: Api) -> None:
     print_results(fail_string, warning)
 
 
+def check_sweeps(api: Api) -> bool:
+    print("Checking sweep creation and agent execution".ljust(72, "."), end="")  # noqa: T201
+    failed_test_strings: List[str] = []
+
+    sweep_config = {
+        "method": "random",
+        "metric": {"goal": "minimize", "name": "score"},
+        "parameters": {
+            "x": {"values": [0.01, 0.05, 0.1]},
+            "y": {"values": [1, 2, 3]},
+        },
+        "name": "verify_sweep",
+    }
+
+    try:
+        with contextlib.redirect_stdout(io.StringIO()):
+            sweep_id = wandb.sweep(
+                sweep=sweep_config, project=PROJECT_NAME, entity=api.default_entity
+            )
+    except Exception as e:
+        failed_test_strings.append(f"Failed to create sweep: {e}")
+        print_results(failed_test_strings, False)
+        return False
+
+    if not sweep_id:
+        failed_test_strings.append("Sweep creation returned an invalid ID.")
+        print_results(failed_test_strings, False)
+        return False
+
+    try:
+
+        def objective(config):
+            score = config.x**3 + config.y
+            return score
+
+        def main():
+            with wandb.init(project=PROJECT_NAME) as run:
+                score = objective(run.config)
+                run.log({"score": score})
+
+        wandb.agent(sweep_id, function=main, count=10)
+    except Exception as e:
+        failed_test_strings.append(f"Failed to run sweep agent: {e}")
+        print_results(failed_test_strings, False)
+        return False
+
+    print_results(failed_test_strings, False)
+    return len(failed_test_strings) == 0
+
+
 def retry_fn(fn: Callable) -> Any:
     ini_time = time.time()
     res = None
