@@ -1,6 +1,4 @@
-"""
-metric.
-"""
+"""metric."""
 
 import logging
 from typing import Callable, Optional, Sequence, Tuple
@@ -11,9 +9,7 @@ logger = logging.getLogger("wandb")
 
 
 class Metric:
-    """
-    Metric object
-    """
+    """Metric object."""
 
     _callback: Optional[Callable[[pb.MetricRecord], None]]
     _name: str
@@ -104,6 +100,8 @@ class Metric:
                 m.summary.none = True
             if "best" in summary_set:
                 m.summary.best = True
+            if "first" in summary_set:
+                m.summary.first = True
         if self._goal == "min":
             m.goal = m.GOAL_MINIMIZE
         if self._goal == "max":
