@@ -1,5 +1,6 @@
 """sync."""
 
+import atexit
 import datetime
 import fnmatch
 import os
@@ -8,19 +9,20 @@ import sys
 import tempfile
 import threading
 import time
+from typing import List, Optional
 from urllib.parse import quote as url_quote
 
 import wandb
 from wandb.proto import wandb_internal_pb2  # type: ignore
 from wandb.sdk.interface.interface_queue import InterfaceQueue
 from wandb.sdk.internal import context, datastore, handler, sender, tb_watcher
+from wandb.sdk.internal.settings_static import SettingsStatic
 from wandb.sdk.lib import filesystem
 from wandb.util import check_and_warn_old
 
 WANDB_SUFFIX = ".wandb"
 SYNCED_SUFFIX = ".synced"
 TFEVENT_SUBSTRING = ".tfevents."
-TMPDIR = tempfile.TemporaryDirectory()
 
 
 class _LocalRun:
@@ -43,6 +45,7 @@ class SyncThread(threading.Thread):
         project=None,
         entity=None,
         run_id=None,
+        job_type=None,
         view=None,
         verbose=None,
         mark_synced=None,
@@ -50,6 +53,8 @@ class SyncThread(threading.Thread):
         sync_tensorboard=None,
         log_path=None,
         append=None,
+        skip_console=None,
+        replace_tags=None,
     ):
         threading.Thread.__init__(self)
         # mark this process as internal
@@ -58,6 +63,7 @@ class SyncThread(threading.Thread):
         self._project = project
         self._entity = entity
         self._run_id = run_id
+        self._job_type = job_type
         self._view = view
         self._verbose = verbose
         self._mark_synced = mark_synced
@@ -65,6 +71,11 @@ class SyncThread(threading.Thread):
         self._sync_tensorboard = sync_tensorboard
         self._log_path = log_path
         self._append = append
+        self._skip_console = skip_console
+        self._replace_tags = replace_tags or {}
+
+        self._tmp_dir = tempfile.TemporaryDirectory()
+        atexit.register(self._tmp_dir.cleanup)
 
     def _parse_pb(self, data, exit_pb=None):
         pb = wandb_internal_pb2.Record()
@@ -72,9 +83,9 @@ class SyncThread(threading.Thread):
         record_type = pb.WhichOneof("record_type")
         if self._view:
             if self._verbose:
-                print("Record:", pb)
+                print("Record:", pb)  # noqa: T201
             else:
-                print("Record:", record_type)
+                print("Record:", record_type)  # noqa: T201
             return pb, exit_pb, True
         if record_type == "run":
             if self._run_id:
@@ -83,7 +94,16 @@ class SyncThread(threading.Thread):
                 pb.run.project = self._project
             if self._entity:
                 pb.run.entity = self._entity
+            if self._job_type:
+                pb.run.job_type = self._job_type
+            # Replace tags if specified
+            if self._replace_tags:
+                new_tags = [self._replace_tags.get(tag, tag) for tag in pb.run.tags]
+                pb.run.ClearField("tags")
+                pb.run.tags.extend(new_tags)
             pb.control.req_resp = True
+        elif record_type in ("output", "output_raw") and self._skip_console:
+            return pb, exit_pb, True
         elif record_type == "exit":
             exit_pb = pb
             return pb, exit_pb, True
@@ -124,7 +144,7 @@ class SyncThread(threading.Thread):
             if tb_event_files > 0 and sync_item.endswith(WANDB_SUFFIX):
                 wandb.termwarn("Found .wandb file, not streaming tensorboard metrics.")
             else:
-                print(f"Found {tb_event_files} tfevent files in {tb_root}")
+                print(f"Found {tb_event_files} tfevent files in {tb_root}")  # noqa: T201
                 if len(tb_logdirs) > 3:
                     wandb.termwarn(
                         f"Found {len(tb_logdirs)} directories containing tfevent files. "
@@ -136,7 +156,7 @@ class SyncThread(threading.Thread):
 
     def _send_tensorboard(self, tb_root, tb_logdirs, send_manager):
         if self._entity is None:
-            viewer, server_info = send_manager._api.viewer_server_info()
+            viewer, _ = send_manager._api.viewer_server_info()
             self._entity = viewer.get("entity")
         proto_run = wandb_internal_pb2.RunRecord()
         proto_run.run_id = self._run_id or wandb.util.generate_id()
@@ -144,13 +164,13 @@ class SyncThread(threading.Thread):
         proto_run.entity = self._entity
         proto_run.telemetry.feature.sync_tfevents = True
 
-        url = "{}/{}/{}/runs/{}".format(
-            self._app_url,
-            url_quote(proto_run.entity),
-            url_quote(proto_run.project),
-            url_quote(proto_run.run_id),
+        url = (
+            f"{self._app_url}"
+            f"/{url_quote(proto_run.entity)}"
+            f"/{url_quote(proto_run.project)}"
+            f"/runs/{url_quote(proto_run.run_id)}"
         )
-        print("Syncing: %s ..." % url)
+        print(f"Syncing: {url} ...")  # noqa: T201
         sys.stdout.flush()
         # using a handler here automatically handles the step
         # logic, adds summaries to the run, and handles different
@@ -168,14 +188,15 @@ class SyncThread(threading.Thread):
         )
         record = send_manager._interface._make_record(run=proto_run)
         settings = wandb.Settings(
-            root_dir=TMPDIR.name,
+            root_dir=self._tmp_dir.name,
             run_id=proto_run.run_id,
-            _start_datetime=datetime.datetime.now(),
-            _start_time=time.time(),
+            x_start_time=time.time(),
         )
 
+        settings_static = SettingsStatic(dict(settings))
+
         handle_manager = handler.HandleManager(
-            settings=settings,
+            settings=settings_static,
             record_q=record_q,
             result_q=None,
             stopped=False,
@@ -186,7 +207,12 @@ class SyncThread(threading.Thread):
 
         filesystem.mkdir_exists_ok(settings.files_dir)
         send_manager.send_run(record, file_dir=settings.files_dir)
-        watcher = tb_watcher.TBWatcher(settings, proto_run, new_interface, True)
+        watcher = tb_watcher.TBWatcher(
+            settings_static,
+            proto_run,
+            new_interface,
+            True,
+        )
 
         for tb in tb_logdirs:
             watcher.add(tb, True, tb_root)
@@ -223,17 +249,15 @@ class SyncThread(threading.Thread):
         except AssertionError as e:
             if ds.in_last_block():
                 wandb.termwarn(
-                    ".wandb file is incomplete ({}), be sure to sync this run again once it's finished".format(
-                        e
-                    )
+                    f".wandb file is incomplete ({e}), be sure to sync this run again once it's finished"
                 )
                 return None
             else:
-                raise e
+                raise
 
     def run(self):
         if self._log_path is not None:
-            print(f"Find logs at: {self._log_path}")
+            print(f"Find logs at: {self._log_path}")  # noqa: T201
         for sync_item in self._sync_list:
             tb_event_files, tb_logdirs, tb_root = self._find_tfevent_files(sync_item)
             if os.path.isdir(sync_item):
@@ -242,7 +266,7 @@ class SyncThread(threading.Thread):
                 if tb_root is None and (
                     check_and_warn_old(files) or len(filtered_files) != 1
                 ):
-                    print(f"Skipping directory: {sync_item}")
+                    print(f"Skipping directory: {sync_item}")  # noqa: T201
                     continue
                 if len(filtered_files) > 0:
                     sync_item = os.path.join(sync_item, filtered_files[0])
@@ -250,10 +274,10 @@ class SyncThread(threading.Thread):
                 tb_root, tb_logdirs, tb_event_files, sync_item
             )
             # If we're syncing tensorboard, let's use a tmp dir for images etc.
-            root_dir = TMPDIR.name if sync_tb else os.path.dirname(sync_item)
+            root_dir = self._tmp_dir.name if sync_tb else os.path.dirname(sync_item)
 
             # When appending we are allowing a possible resume, ie the run
-            # doesnt have to exist already
+            # does not have to exist already
             resume = "allow" if self._append else None
 
             sm = sender.SendManager.setup(root_dir, resume=resume)
@@ -265,7 +289,7 @@ class SyncThread(threading.Thread):
             try:
                 ds.open_for_scan(sync_item)
             except AssertionError as e:
-                print(f".wandb file is empty ({e}), skipping: {sync_item}")
+                print(f".wandb file is empty ({e}), skipping: {sync_item}")  # noqa: T201
                 continue
 
             # save exit for final send
@@ -293,13 +317,13 @@ class SyncThread(threading.Thread):
                     if not shown and result_type == "run_result":
                         r = result.run_result.run
                         # TODO(jhr): hardcode until we have settings in sync
-                        url = "{}/{}/{}/runs/{}".format(
-                            self._app_url,
-                            url_quote(r.entity),
-                            url_quote(r.project),
-                            url_quote(r.run_id),
+                        url = (
+                            f"{self._app_url}"
+                            f"/{url_quote(r.entity)}"
+                            f"/{url_quote(r.project)}"
+                            f"/runs/{url_quote(r.run_id)}"
                         )
-                        print("Syncing: %s ... " % url, end="")
+                        print(f"Syncing: {url} ... ", end="")  # noqa: T201
                         sys.stdout.flush()
                         shown = True
             sm.finish()
@@ -308,7 +332,7 @@ class SyncThread(threading.Thread):
                 synced_file = f"{sync_item}{SYNCED_SUFFIX}"
                 with open(synced_file, "w"):
                     pass
-            print("done.")
+            print("done.")  # noqa: T201
 
 
 class SyncManager:
@@ -317,6 +341,7 @@ class SyncManager:
         project=None,
         entity=None,
         run_id=None,
+        job_type=None,
         mark_synced=None,
         app_url=None,
         view=None,
@@ -324,12 +349,15 @@ class SyncManager:
         sync_tensorboard=None,
         log_path=None,
         append=None,
+        skip_console=None,
+        replace_tags=None,
     ):
         self._sync_list = []
         self._thread = None
         self._project = project
         self._entity = entity
         self._run_id = run_id
+        self._job_type = job_type
         self._mark_synced = mark_synced
         self._app_url = app_url
         self._view = view
@@ -337,6 +365,8 @@ class SyncManager:
         self._sync_tensorboard = sync_tensorboard
         self._log_path = log_path
         self._append = append
+        self._skip_console = skip_console
+        self._replace_tags = replace_tags or {}
 
     def status(self):
         pass
@@ -351,6 +381,7 @@ class SyncManager:
             project=self._project,
             entity=self._entity,
             run_id=self._run_id,
+            job_type=self._job_type,
             view=self._view,
             verbose=self._verbose,
             mark_synced=self._mark_synced,
@@ -358,6 +389,8 @@ class SyncManager:
             sync_tensorboard=self._sync_tensorboard,
             log_path=self._log_path,
             append=self._append,
+            skip_console=self._skip_console,
+            replace_tags=self._replace_tags,
         )
         self._thread.start()
 
@@ -370,17 +403,15 @@ class SyncManager:
 
 
 def get_runs(
-    include_offline=None,
-    include_online=None,
-    include_synced=None,
-    include_unsynced=None,
-    exclude_globs=None,
-    include_globs=None,
+    include_offline: bool = True,
+    include_online: bool = True,
+    include_synced: bool = False,
+    include_unsynced: bool = True,
+    exclude_globs: Optional[List[str]] = None,
+    include_globs: Optional[List[str]] = None,
 ):
     # TODO(jhr): grab dir info from settings
-    base = "wandb"
-    if os.path.exists(".wandb"):
-        base = ".wandb"
+    base = ".wandb" if os.path.exists(".wandb") else "wandb"
     if not os.path.exists(base):
         return ()
     all_dirs = os.listdir(base)
