@@ -1,7 +1,6 @@
 """Compatibility wandb_run module.
 
-In the future use:
-    from wandb.sdk.wandb_run import Run
+Please use `wandb.Run` instead.
 """
 
 from wandb.sdk.wandb_run import Run
